#################################################################################################################
# Name:    mx42-configure-peer-download.ps1
# Created: 15.11.2019
# Updated: 28.11.2019
#
# Copyrights by Matrix42 AG
#
#
# The PowerShell script installs BranchCache if required.
#
# Example call : .\mx42-configure-peer-download.ps1  -LogFile '.\Subdepot.log'
#
# Version: 1.0
#
#################################################################################################################

param([parameter(Mandatory=$FALSE,Position=0)]
	  [String] $LogFile=".\Configure-Peer-Download.log"
)

#################################################################################################################

. ".\mx42-common.ps1"

StartCapture $LogFile


function show_params
{
    Write-Host "Script mx42-configure-peer-download.ps1"
    Write-Host "Given Parameters:"
    Write-Host "LogFile....................: $LogFile"
}

# Import the server manager
Import-Module servermanager

# Log parameters
show_params


#################################################################################################################
# PRECONDITION -> PowerShell Group Policy Management
#################################################################################################################

# Check if Group Policy Management is installed. If not install it
$GPMCIsInstalled = (Get-WindowsFeature GPMC).Installed
if($GPMCIsInstalled -eq $false)
{
	Write-Host "Install group policy management console for PowerShell"
	Install-WindowsFeature GPMC
}

# Refresh Group policies
gpupdate /force

# Check Group policy
$AppliedGroupPolicies = (Get-GPO -All).DisplayName
# Iterate through all policies and check if the one for BranchCache exists
$BranchCachePolicyApplied = $false
foreach ($policy in $AppliedGroupPolicies){
	Write-Host "Detected group policy:" $policy
	if($policy -match 'BranchCache')
	{
		$BranchCachePolicyApplied = $true
	}
}
# Check if BranchCache policy is applied
if($BranchCachePolicyApplied -eq $false)
{
	Write-Host "Error: No group policy with 'BranchCache' in its name was found. Please check your group policies applied to this server."
	Write-Host "Error: Abort installation..."
	exit 10
}
else
{
	Write-Host "The expected group policy was found. Continue with installation..."
}


#################################################################################################################
# INSTALL SERVERROLE
#################################################################################################################

# Check if BranchCache feature is installed. If not install it
$BranchCacheFeatureIsInstalled = (Get-WindowsFeature BranchCache).Installed
if($BranchCacheFeatureIsInstalled -eq $false)
{
	Write-Host "The BranchCache feature is not installed on this machine. Continue with installation..."
	Install-WindowsFeature BranchCache
}else{
	Write-Host "The BranchCache feature is already installed on this machine. Skip installation..."
}

# Check if BranchCache file service is installed. If not install it
$BranchCacheFileServiceIsInstalled = (Get-WindowsFeature FS-BranchCache).Installed
if($BranchCacheFileServiceIsInstalled -eq $false)
{
	Write-Host "The BranchCache file service is not installed on this machine. Continue with installation..."
	Install-WindowsFeature FS-BranchCache
}else{
	Write-Host "The BranchCache file service is already installed on this machine. Skip installation..."
}


#################################################################################################################
# Enable Firewall rules and configure BranchCache
#################################################################################################################
Enable-NetFirewallRule -Name Microsoft-Windows-PeerDist-HttpTrans-In
Enable-NetFirewallRule -Name Microsoft-Windows-PeerDist-HttpTrans-Out
Enable-NetFirewallRule -Name Microsoft-Windows-PeerDist-WSD-In
Enable-NetFirewallRule -Name Microsoft-Windows-PeerDist-WSD-Out


#################################################################################################################
# Enable Configurator$ for BranchCache
#################################################################################################################
net share "Configurator$" /Cache:BranchCache


#################################################################################################################
# Check that BranchCache is installed
#################################################################################################################
Write-Host "Check BranchCache"
$BranchCacheIsEnabled = (Get-bcstatus).BranchCacheIsEnabled
if($BranchCacheIsEnabled -eq $true)
{
	Write-Host -ForegroundColor Green "BranchCache configured successfully"
	exit 0
}else
{
	Write-Host "Error: BranchCache was not installed correctly"
	exit 1
}


#################################################################################################################


# Catch exceptions

trap [Exception]
{
    Write-Host -ForegroundColor Red "Error: $($_.Exception.Message)"
	exit 1
}

# SIG # Begin signature block
# MIIYEQYJKoZIhvcNAQcCoIIYAjCCF/4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCA6RMDBinkbU0Ka
# DGvPlCgmkyXr7WDE/iL/eVmwN9Jzh6CCEsgwggPuMIIDV6ADAgECAhB+k+v7fMZO
# WepLmnfUBvw7MA0GCSqGSIb3DQEBBQUAMIGLMQswCQYDVQQGEwJaQTEVMBMGA1UE
# CBMMV2VzdGVybiBDYXBlMRQwEgYDVQQHEwtEdXJiYW52aWxsZTEPMA0GA1UEChMG
# VGhhd3RlMR0wGwYDVQQLExRUaGF3dGUgQ2VydGlmaWNhdGlvbjEfMB0GA1UEAxMW
# VGhhd3RlIFRpbWVzdGFtcGluZyBDQTAeFw0xMjEyMjEwMDAwMDBaFw0yMDEyMzAy
# MzU5NTlaMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAsayzSVRLlxwS
# CtgleZEiVypv3LgmxENza8K/LlBa+xTCdo5DASVDtKHiRfTot3vDdMwi17SUAAL3
# Te2/tLdEJGvNX0U70UTOQxJzF4KLabQry5kerHIbJk1xH7Ex3ftRYQJTpqr1SSwF
# eEWlL4nO55nn/oziVz89xpLcSvh7M+R5CvvwdYhBnP/FA1GZqtdsn5Nph2Upg4XC
# YBTEyMk7FNrAgfAfDXTekiKryvf7dHwn5vdKG3+nw54trorqpuaqJxZ9YfeYcRG8
# 4lChS+Vd+uUOpyyfqmUg09iW6Mh8pU5IRP8Z4kQHkgvXaISAXWp4ZEXNYEZ+VMET
# fMV58cnBcQIDAQABo4H6MIH3MB0GA1UdDgQWBBRfmvVuXMzMdJrU3X3vP9vsTIAu
# 3TAyBggrBgEFBQcBAQQmMCQwIgYIKwYBBQUHMAGGFmh0dHA6Ly9vY3NwLnRoYXd0
# ZS5jb20wEgYDVR0TAQH/BAgwBgEB/wIBADA/BgNVHR8EODA2MDSgMqAwhi5odHRw
# Oi8vY3JsLnRoYXd0ZS5jb20vVGhhd3RlVGltZXN0YW1waW5nQ0EuY3JsMBMGA1Ud
# JQQMMAoGCCsGAQUFBwMIMA4GA1UdDwEB/wQEAwIBBjAoBgNVHREEITAfpB0wGzEZ
# MBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMTANBgkqhkiG9w0BAQUFAAOBgQADCZuP
# ee9/WTCq72i1+uMJHbtPggZdN1+mUp8WjeockglEbvVt61h8MOj5aY0jcwsSb0ep
# rjkR+Cqxm7Aaw47rWZYArc4MTbLQMaYIXCp6/OJ6HVdMqGUY6XlAYiWWbsfHN2qD
# IQiOQerd2Vc/HXdJhyoWBl6mOGoiEqNRGYN+tjCCBKMwggOLoAMCAQICEA7P9DjI
# /r81bgTYapgbGlAwDQYJKoZIhvcNAQEFBQAwXjELMAkGA1UEBhMCVVMxHTAbBgNV
# BAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBUaW1l
# IFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzIwHhcNMTIxMDE4MDAwMDAwWhcNMjAx
# MjI5MjM1OTU5WjBiMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29y
# cG9yYXRpb24xNDAyBgNVBAMTK1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2Vydmlj
# ZXMgU2lnbmVyIC0gRzQwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCi
# Yws5RLi7I6dESbsO/6HwYQpTk7CY260sD0rFbv+GPFNVDxXOBD8r/amWltm+YXkL
# W8lMhnbl4ENLIpXuwitDwZ/YaLSOQE/uhTi5EcUj8mRY8BUyb05Xoa6IpALXKh7N
# S+HdY9UXiTJbsF6ZWqidKFAOF+6W22E7RVEdzxJWC5JH/Kuu9mY9R6xwcueS51/N
# ELnEg2SUGb0lgOHo0iKl0LoCeqF3k1tlw+4XdLxBhircCEyMkoyRLZ53RB9o1qh0
# d9sOWzKLVoszvdljyEmdOsXF6jML0vGjG/SLvtmzV4s73gSneiKyJK4ux3DFvk6D
# Jgj7C72pT5kI4RAocqrNAgMBAAGjggFXMIIBUzAMBgNVHRMBAf8EAjAAMBYGA1Ud
# JQEB/wQMMAoGCCsGAQUFBwMIMA4GA1UdDwEB/wQEAwIHgDBzBggrBgEFBQcBAQRn
# MGUwKgYIKwYBBQUHMAGGHmh0dHA6Ly90cy1vY3NwLndzLnN5bWFudGVjLmNvbTA3
# BggrBgEFBQcwAoYraHR0cDovL3RzLWFpYS53cy5zeW1hbnRlYy5jb20vdHNzLWNh
# LWcyLmNlcjA8BgNVHR8ENTAzMDGgL6AthitodHRwOi8vdHMtY3JsLndzLnN5bWFu
# dGVjLmNvbS90c3MtY2EtZzIuY3JsMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBU
# aW1lU3RhbXAtMjA0OC0yMB0GA1UdDgQWBBRGxmmjDkoUHtVM2lJjFz9eNrwN5jAf
# BgNVHSMEGDAWgBRfmvVuXMzMdJrU3X3vP9vsTIAu3TANBgkqhkiG9w0BAQUFAAOC
# AQEAeDu0kSoATPCPYjA3eKOEJwdvGLLeJdyg1JQDqoZOJZ+aQAMc3c7jecshaAba
# tjK0bb/0LCZjM+RJZG0N5sNnDvcFpDVsfIkWxumy37Lp3SDGcQ/NlXTctlzevTcf
# Q3jmeLXNKAQgo6rxS8SIKZEOgNER/N1cdm5PXg5FRkFuDbDqOJqxOtoJcRD8HHm0
# gHusafT9nLYMFivxf1sJPZtb4hbKE4FtAC44DagpjyzhsvRaqQGvFZwsL0kb2yK7
# w/54lFHDhrGCiF3wPbRRoXkzKy57udwgCRNx62oZW8/opTBXLIlJP7nPf8m/PiJo
# Y1OavWl0rMUdPH+S4MO8HNgEdTCCBM4wggO2oAMCAQICED3L9ZzvVRhJCatCMCxN
# ktQwDQYJKoZIhvcNAQELBQAwfzELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFu
# dGVjIENvcnBvcmF0aW9uMR8wHQYDVQQLExZTeW1hbnRlYyBUcnVzdCBOZXR3b3Jr
# MTAwLgYDVQQDEydTeW1hbnRlYyBDbGFzcyAzIFNIQTI1NiBDb2RlIFNpZ25pbmcg
# Q0EwHhcNMTgwMzE1MDAwMDAwWhcNMjEwNTEyMjM1OTU5WjBmMQswCQYDVQQGEwJE
# RTEPMA0GA1UECAwGSGVzc2VuMRowGAYDVQQHDBFGcmFua2Z1cnQgYW0gTWFpbjEU
# MBIGA1UECgwLTWF0cml4NDIgQUcxFDASBgNVBAMMC01hdHJpeDQyIEFHMIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwWJCiaJ1Ji13WoMVZb2z9NhHjp4k
# tZGkzetOBTFD3ZZdQCAAe9h2QG/HLpGi5r7uZCn+B5NK5GhBXKx+xM9rKWrhDtS4
# 9L6f7pDTwarMiCGI+O+kQd8EdLj5/0xCgoH2ee4eHTtagEki3eibnHzVPYDH5nRl
# Zlo6mChRtVFAJoZFH2MvbQumKh3YV57xw8/ZWb/2ZBX+S/M5E1wVN9FyUadrSknK
# b9Tt6d60ZnOEVGcHUcudUQZIhZt8PhiAe78u7dkw8uHgcXikauemLfs0u6+jVUtd
# /hBl4oUtJDGFe5dfejXBOjwiUGCsPH+RkgjKR7P3rXaAwvZiWmgUKRD0SQIDAQAB
# o4IBXTCCAVkwCQYDVR0TBAIwADAOBgNVHQ8BAf8EBAMCB4AwKwYDVR0fBCQwIjAg
# oB6gHIYaaHR0cDovL3N2LnN5bWNiLmNvbS9zdi5jcmwwYQYDVR0gBFowWDBWBgZn
# gQwBBAEwTDAjBggrBgEFBQcCARYXaHR0cHM6Ly9kLnN5bWNiLmNvbS9jcHMwJQYI
# KwYBBQUHAgIwGQwXaHR0cHM6Ly9kLnN5bWNiLmNvbS9ycGEwEwYDVR0lBAwwCgYI
# KwYBBQUHAwMwVwYIKwYBBQUHAQEESzBJMB8GCCsGAQUFBzABhhNodHRwOi8vc3Yu
# c3ltY2QuY29tMCYGCCsGAQUFBzAChhpodHRwOi8vc3Yuc3ltY2IuY29tL3N2LmNy
# dDAfBgNVHSMEGDAWgBSWO1PweTOXr32D7y4rzMq3hh5yZjAdBgNVHQ4EFgQUQm1H
# y/FoRTLQkgRhUUTEyrwxzD4wDQYJKoZIhvcNAQELBQADggEBAFj1SKiiu0eqW/E/
# s9DxT/O9thl0ZOjlFH+2cVBwuougWdZXz/xDPh4u4x0cffqtftSuEX388b1jm+6v
# NnTFzxXO11DurLaG6YO3A+Oau65HM9v7GN/VdJPCV+ChIcWcKcnrmMT5A5YoPFLH
# vvj55Zf0agP9Dpy1emufXJPIwQnts744QGIeco4ixX/DNnsrRGAaW6eKyNTTEFtX
# 5ETi6Y3gztPHgxrDh20Zg+mDVvqtWCCNY3xB3m87jZolhLgE4by5+wdjORgno95e
# HTaKcThb6xx552ULS5PkSk7x1ZJTztlxSEMNr/a8jevCguMPpWld1oCica2N+Z49
# /AgJ/VcwggVZMIIEQaADAgECAhA9eNf5dklgsmF99PAeyoYqMA0GCSqGSIb3DQEB
# CwUAMIHKMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAd
# BgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOjA4BgNVBAsTMShjKSAyMDA2
# IFZlcmlTaWduLCBJbmMuIC0gRm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxRTBDBgNV
# BAMTPFZlcmlTaWduIENsYXNzIDMgUHVibGljIFByaW1hcnkgQ2VydGlmaWNhdGlv
# biBBdXRob3JpdHkgLSBHNTAeFw0xMzEyMTAwMDAwMDBaFw0yMzEyMDkyMzU5NTla
# MH8xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEf
# MB0GA1UECxMWU3ltYW50ZWMgVHJ1c3QgTmV0d29yazEwMC4GA1UEAxMnU3ltYW50
# ZWMgQ2xhc3MgMyBTSEEyNTYgQ29kZSBTaWduaW5nIENBMIIBIjANBgkqhkiG9w0B
# AQEFAAOCAQ8AMIIBCgKCAQEAl4MeABavLLHSCMTXaJNRYB5x9uJHtNtYTSNiarS/
# WhtR96MNGHdou9g2qy8hUNqe8+dfJ04LwpfICXCTqdpcDU6kDZGgtOwUzpFyVC7O
# o9tE6VIbP0E8ykrkqsDoOatTzCHQzM9/m+bCzFhqghXuPTbPHMWXBySO8Xu+MS09
# bty1mUKfS2GVXxxw7hd924vlYYl4x2gbrxF4GpiuxFVHU9mzMtahDkZAxZeSitFT
# p5lbhTVX0+qTYmEgCscwdyQRTWKDtrp7aIIx7mXK3/nVjbI13Iwrb2pyXGCEnPIM
# lF7AVlIASMzT+KV93i/XE+Q4qITVRrgThsIbnepaON2b2wIDAQABo4IBgzCCAX8w
# LwYIKwYBBQUHAQEEIzAhMB8GCCsGAQUFBzABhhNodHRwOi8vczIuc3ltY2IuY29t
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwbAYDVR0gBGUwYzBhBgtghkgBhvhFAQcXAzBS
# MCYGCCsGAQUFBwIBFhpodHRwOi8vd3d3LnN5bWF1dGguY29tL2NwczAoBggrBgEF
# BQcCAjAcGhpodHRwOi8vd3d3LnN5bWF1dGguY29tL3JwYTAwBgNVHR8EKTAnMCWg
# I6Ahhh9odHRwOi8vczEuc3ltY2IuY29tL3BjYTMtZzUuY3JsMB0GA1UdJQQWMBQG
# CCsGAQUFBwMCBggrBgEFBQcDAzAOBgNVHQ8BAf8EBAMCAQYwKQYDVR0RBCIwIKQe
# MBwxGjAYBgNVBAMTEVN5bWFudGVjUEtJLTEtNTY3MB0GA1UdDgQWBBSWO1PweTOX
# r32D7y4rzMq3hh5yZjAfBgNVHSMEGDAWgBR/02Wnwt3su/AwCfNDOfoCrzMxMzAN
# BgkqhkiG9w0BAQsFAAOCAQEAE4UaHmmpN/egvaSvfh1hU/6djF4MpnUeeBcj3f3s
# GgNVOftxlcdlWqeOMNJEWmHbcG/aIQXCLnO6SfHRk/5dyc1eA+CJnj90Htf3OIup
# 1s+7NS8zWKiSVtHITTuC5nmEFvwosLFH8x2iPu6H2aZ/pFalP62ELinefLyoqqM9
# BAHqupOiDlAiKRdMh+Q6EV/WpCWJmwVrL7TJAUwnewusGQUioGAVP9rJ+01Mj/ty
# Z3f9J5THujUOiEn+jf0or0oSvQ2zlwXeRAwV+jYrA9zBUAHxoRFdFOXivSdLVL4r
# hF4PpsN0BQrvl8OJIrEfd/O9zUPU8UypP7WLhK9k8tAUITGCBJ8wggSbAgEBMIGT
# MH8xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEf
# MB0GA1UECxMWU3ltYW50ZWMgVHJ1c3QgTmV0d29yazEwMC4GA1UEAxMnU3ltYW50
# ZWMgQ2xhc3MgMyBTSEEyNTYgQ29kZSBTaWduaW5nIENBAhA9y/Wc71UYSQmrQjAs
# TZLUMA0GCWCGSAFlAwQCAQUAoIHOMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEE
# MBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCCF
# OikFvfRTvs2lEE0Yd/ygDhjAjKAvPWewECiq2eYT9TBiBgorBgEEAYI3AgEMMVQw
# UqA2gDQATQBhAHQAcgBpAHgANAAyACAAQwBsAGkAZQBuAHQAIABNAGEAbgBhAGcA
# ZQBtAGUAbgB0oRiAFmh0dHA6Ly93d3cubWF0cml4NDIuZGUwDQYJKoZIhvcNAQEB
# BQAEggEARECS3RFo+LdvolcjKhtKOeY6NsKfuBjGOhNcHzuW5ZhGOT3ocDTA90o9
# ql0t5+N0FYTpZGWLY9PSZEgNlYakqNeZUS30UoCDPhBgmeIRx+AGMxzyAksqiW1j
# pfjntiHd4s220XHkiuwiYRybc95YsA+jPLl8FWW73Avfssq1ee3EzQJZtzFtm7Wg
# 8mZ9qFXo0gJlqVxN+MRmk3a1XOL3+v+KK4E41k1rpIvto06ew2WfKNjzl35ST9H7
# 562pEfNP6KZU6Gk6bQz3x/UPLvlEo2V30bvXYvLNY3N7wf85bczbPJXPkTRNZgSW
# 4cuy9gi887NSp12YIqIS3Zxzl8z6KaGCAgswggIHBgkqhkiG9w0BCQYxggH4MIIB
# 9AIBATByMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyAhAOz/Q4yP6/NW4E2GqYGxpQMAkGBSsOAwIaBQCgXTAYBgkqhkiG9w0B
# CQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0xOTEyMDIxMzQ3MTNaMCMG
# CSqGSIb3DQEJBDEWBBQC2uvBFG06kQzSHVuo11g9pPx4RDANBgkqhkiG9w0BAQEF
# AASCAQA5JNXQuHj9/8S1TGdBiRce1ccmFRktCz6JbWCWwRKXPoDOuTLK6DQuofMp
# yNL9NB6XD9KQul3wvLk0KKDL7L6VG+XWhO1QBCdzdJbP/JanjaTvXzWrC4Su8rW3
# NcOhq8rXYTYu8cBdZrv/VwwYmsUQkgnt7f/q+iRgnq1APKuFcVWIuSklzvlrIxG+
# EIcXOPrDmGQ8pVJuuHH4yxe11TDa/7yQjb1CjpgySltO7dUJz2DWH4LcLK8L/O8k
# usu7DN5QUvtbkb2acvGyDdy7dEgNDHZVzkz7t4R/N2JkWT6waVoIATQTgPMZmkW1
# esWP6EBPA8xQA5sPU5T486T149iO
# SIG # End signature block
